import os

from telegram import Bot, ParseMode
from telegram import ReplyKeyboardMarkup, ReplyKeyboardRemove
from telegram import InlineKeyboardButton, InlineKeyboardMarkup
from telegram.ext import Updater,Dispatcher
from telegram.ext import CommandHandler, ConversationHandler
from telegram.ext import CallbackQueryHandler, MessageHandler, Filters
from telegram import Update
from telegram.ext import (
    BasePersistence,
    CallbackContext,

    ChosenInlineResultHandler,
    DispatcherHandlerStop,
    Handler,
    InlineQueryHandler,
)
from telegram.ext.utils.types import CCT

from telegram import InlineQueryResultArticle, InputTextMessageContent
#from telegram.ext import InlineQueryHandler

from typing import TYPE_CHECKING, Dict, List, NoReturn, Optional, Union, Tuple, cast, ClassVar


from RenePy.Utiles import *



NumeroOpcionActual:int=1000
def getNumeroOpcionActual():
    global NumeroOpcionActual
    NumeroOpcionActual=NumeroOpcionActual+1
    return(NumeroOpcionActual+0)

class IAccionComando:
    def __init__(self,metodoComando=None,opcionComando:int=None):
        if opcionComando == None:
            opcionComando = getNumeroOpcionActual()

        self.opcionComando:int=opcionComando
        self.metodoComando=metodoComando
class ITextAccionComando(IAccionComando):
    def __init__(self, texto: str, metodoComando=None, opcionComando: int = None):
        self.texto: str = texto
        IAccionComando.__init__(self, metodoComando=metodoComando, opcionComando=opcionComando)

class IEncadenados:
    def __init__(self,encadenados:List=None):
        if encadenados==None:
            encadenados=[]
        self.encadenados:List=encadenados
    def hayEncadenados(self):
        return len(self.encadenados)>0

class BotonDeMenuC(ITextAccionComando):
    def __init__(self, textoDeboton:str,metodoComando=None,opcionComando:int=None):
        self.textoDeboton:str=textoDeboton
        ITextAccionComando.__init__(self,texto=textoDeboton,metodoComando=metodoComando,opcionComando=opcionComando)

class ComandoEnMenu(ITextAccionComando):
    def __init__(self, textoDeComando:str,metodoComando=None,opcionComando:int=None):
        self.textoDeboton:str=textoDeComando
        ITextAccionComando.__init__(self,texto=textoDeComando,metodoComando=metodoComando,opcionComando=opcionComando)
TEXTO_EN_MENU_DEFAULT="Selecciona una opcion del menu"
class Teclado(IEncadenados):
    def __init__(self,menuSoloTextoEnBotones,textoDeMenu=None,encadenados:List=None):
        if menuSoloTextoEnBotones is None:
            menuSoloTextoEnBotones=[]
        if textoDeMenu is None:
            textoDeMenu=TEXTO_EN_MENU_DEFAULT
        self.textoDeMenu=textoDeMenu
        self.__menuSoloTextoEnBotones = menuSoloTextoEnBotones
        IEncadenados.__init__(self, encadenados=encadenados)
    def getMenuSoloTextoEnBotones(self,update=None,ctx=None):
        if esFuncion(self.__menuSoloTextoEnBotones):
            return self.__menuSoloTextoEnBotones(update,ctx)
        return self.__menuSoloTextoEnBotones
    def tieneBotones(self):
        return len(self.getMenuSoloTextoEnBotones())>0
    def getTextoDeMenu(self,update=None,ctx=None):
        if esFuncion(self.textoDeMenu):
            return self.textoDeMenu(update,ctx)
        return self.textoDeMenu

    def addTextoAlMenuSoloTextoEnBotones(self,texto:str):
        getMenuSoloTextoEnBotones = self.getMenuSoloTextoEnBotones
        def getMenuSoloTextoEnBotonesAgregado(update=None,ctx=None):
            menu=getMenuSoloTextoEnBotones(update,ctx)
            if not contiene(menu,[texto]):
                menu.append([texto])
            return menu
        self.getMenuSoloTextoEnBotones=getMenuSoloTextoEnBotonesAgregado



class MenuDeBotonesC(Teclado):
    def __init__(self, matrizDeBotones,textoDeMenu=None,comandosInternos=None,encadenados:List=None):
        """

        :param matrizDeBotones:  List[List[BotonDeMenuC]] o function(update,ctx)
        :param textoDeMenu:
        :param comandosInternos: List[ComandoEnMenu] o function(update,ctx)
        """
        #:List[List[BotonDeMenuC]]
        #:List[ComandoEnMenu]
        if comandosInternos==None:
            comandosInternos=[]
        self.__matrizDeBotones = matrizDeBotones

        def getMenuSoloTextoEnBotones( update=None, ctx=None):
            menuSoloTextoEnBotones: List[List[str]] = []
            if esFuncion(matrizDeBotones):
                submatriz = matrizDeBotones(update, ctx)
            else:
                submatriz = matrizDeBotones
            for filaBotones in submatriz:
                filaTemporalDeTexoEnBotones: List[str] = []
                for botonDeMenu in filaBotones:
                    filaTemporalDeTexoEnBotones.append(botonDeMenu.textoDeboton)
                menuSoloTextoEnBotones.append(filaTemporalDeTexoEnBotones)
            return menuSoloTextoEnBotones
        super().__init__(menuSoloTextoEnBotones=getMenuSoloTextoEnBotones,textoDeMenu=textoDeMenu,encadenados=encadenados)



        self.__comandosInternos=comandosInternos
    def getComandosInternos(self):#,update=None,ctx=None
        # if esFuncion(self.__comandosInternos):
        #     comandosInternos=self.__comandosInternos(update,ctx)
        # else:
        #     comandosInternos=self.__comandosInternos
        # return comandosInternos
        return self.__comandosInternos
    def getListaDeBotones(self,update=None,ctx=None):
        listaDeBotones: List[BotonDeMenuC] = []
        if esFuncion(self.__matrizDeBotones):
            matrizDeBotones=self.__matrizDeBotones(update,ctx)
        else:
            matrizDeBotones=self.__matrizDeBotones
        for filaBotones in matrizDeBotones:
            #filaTemporalDeTexoEnBotones: List[str] = []
            for botonDeMenu in filaBotones:
                #filaTemporalDeTexoEnBotones.append(botonDeMenu.textoDeboton)
                listaDeBotones.append(botonDeMenu)
        return listaDeBotones

    def addBoton(self,boton:BotonDeMenuC):


        getMenuSoloTextoEnBotones=self.getMenuSoloTextoEnBotones
        def getMenuSoloTextoEnBotonesAgregado(update=None,ctx=None):
            menu=getMenuSoloTextoEnBotones(update,ctx)
            menu.append([boton.textoDeboton])
            return menu
        self.getMenuSoloTextoEnBotones=getMenuSoloTextoEnBotonesAgregado

        getListaDeBotones=self.getListaDeBotones
        def getListaDeBotonesAgregado(update=None,ctx=None):
            menu = getListaDeBotones(update, ctx)
            menu.append(boton)
            return menu
        self.getListaDeBotones=getListaDeBotonesAgregado




        #self.menuSoloTextoEnBotones.append([boton.textoDeboton])
        #self.listaDeBotones.append(boton)
        # self.matrizDeBotones.append([boton])


class Inputt(Teclado):#IAccionComando
    def __init__(self,metodoComando,textoDeMenu,menuSoloTextoEnBotones=None,ponerBotonCancelar:bool=True,encadenados:List=None,tipo=Filters.text):#opcionComando:int=None
        """

        :param metodoComando:
        :param textoDeMenu:
        :param menuSoloTextoEnBotones: List[List[str]] o function(update,ctx)
        :param ponerBotonCancelar:
        :param encadenados:
        :param tipo:
        """
        #:List[List[str]]
        if menuSoloTextoEnBotones==None:
            menuSoloTextoEnBotones=[]
        super().__init__(menuSoloTextoEnBotones=menuSoloTextoEnBotones,textoDeMenu=textoDeMenu,encadenados=encadenados)
        #IEncadenados.__init__(self,encadenados=encadenados)
        self.metodoComando=metodoComando
        self.ponerBotonCancelar:bool=ponerBotonCancelar
        # if encadenados==None:
        #     encadenados=[]
        # self.encadenados:List=encadenados
        self.tipo=tipo

    # def hayEnencadenadosdos(self):
    #     return len(self.encadenados)>0

class IParaInputt:
    def __init__(self,inputt:Inputt):
        self.inputt=inputt

class ITieneMenu:
    def __init__(self,menuDeBotonesC:MenuDeBotonesC):
        self.menuDeBotonesC: MenuDeBotonesC = menuDeBotonesC

class _BotonMenuInicial(BotonDeMenuC,ITieneMenu):
    def __init__(self, botonMenuInicial:BotonDeMenuC,menuDeBotonesC:MenuDeBotonesC):
        self.botonMenuInicial:BotonDeMenuC=botonMenuInicial
        #self.menuDeBotonesC:MenuDeBotonesC=menuDeBotonesC
        super().__init__(textoDeboton=botonMenuInicial.textoDeboton,metodoComando=botonMenuInicial.metodoComando,opcionComando=botonMenuInicial.opcionComando)
        ITieneMenu.__init__(self,menuDeBotonesC=menuDeBotonesC)

class BotonParaMenu(BotonDeMenuC,ITieneMenu):
    def __init__(self,textoDeboton:str,menuDeBotonesC:MenuDeBotonesC,metodoComando=None,opcionComando:int=None):
        # if opcionComando == None:
        #     opcionComando = getNumeroOpcionActual()
        super().__init__(textoDeboton=textoDeboton,metodoComando=metodoComando,opcionComando=opcionComando)
        #self.menuDeBotonesC:MenuDeBotonesC=menuDeBotonesC
        ITieneMenu.__init__(self, menuDeBotonesC=menuDeBotonesC)
class ComandoParaMenu(ComandoEnMenu,ITieneMenu):
    def __init__(self,textoDeComando:str,menuDeBotonesC:MenuDeBotonesC,metodoComando=None,opcionComando:int=None):
        # if opcionComando == None:
        #     opcionComando = getNumeroOpcionActual()
        super().__init__(textoDeComando=textoDeComando,metodoComando=metodoComando,opcionComando=opcionComando)
        ITieneMenu.__init__(self, menuDeBotonesC=menuDeBotonesC)
class BotonParaInputt(BotonDeMenuC,IParaInputt):
    def __init__(self,textoDeboton:str,inputt:Inputt,metodoComando=None,opcionComando:int=None):
        super().__init__(textoDeboton=textoDeboton, metodoComando=metodoComando, opcionComando=opcionComando)
        IParaInputt.__init__(self,inputt=inputt)
class ComandoParaInputt(ComandoEnMenu,IParaInputt):
    def __init__(self,textoDeComando:str,inputt:Inputt,metodoComando=None,opcionComando:int=None):
        super().__init__(textoDeComando=textoDeComando, metodoComando=metodoComando, opcionComando=opcionComando)
        IParaInputt.__init__(self,inputt=inputt)
class AccionParaInputt(IAccionComando,IParaInputt):
    def __init__(self,inputt:Inputt,metodoComando=None,opcionComando:int=None):
        IAccionComando.__init__(self, metodoComando=metodoComando, opcionComando=opcionComando)
        IParaInputt.__init__(self,inputt=inputt)

class AccionParaMenu(IAccionComando,ITieneMenu):
    def __init__(self,menuDeBotonesC:MenuDeBotonesC,metodoComando=None,opcionComando:int=None):
        super().__init__(metodoComando=metodoComando,opcionComando=opcionComando)
        ITieneMenu.__init__(self, menuDeBotonesC=menuDeBotonesC)

class BotonRetroceder(BotonDeMenuC):
    def __init__(self,textoDeboton:str,parent,metodoComando=None):
        super().__init__(textoDeboton=textoDeboton, metodoComando=None, opcionComando=parent.opcionComando)
        if isinstance(parent,ITieneMenu):
            self.menuDeBotonesC: MenuDeBotonesC = parent.menuDeBotonesC
        else:
        #elif isinstance(IParaInputt):
            self.menuDeBotonesC =None#parent.inputt.
        self.parent=parent


class UsuarioDeTelegram:
    def __init__(self,id,first_name,last_name,username,is_bot,link  ):
        self.first_name:str=first_name
        self.id:int=id
        self.last_name:str=last_name
        self.username:str=username
        self.is_bot:bool=is_bot
        self.link = link
class ChatDeTelegram:
    def __init__(self, id, type, title, first_name,last_name,username,invite_link,linked_chat_id,link):
        self.id=id
        self.type = type
        self.title = title
        self.first_name = first_name
        self.last_name = last_name
        self.username = username
        self.invite_link = invite_link
        self.linked_chat_id = linked_chat_id
        self.link = link
class DocumentoDeTelgram:
    def __init__(self, file_id, file_unique_id , file_name , mime_type , file_size ):
        self.file_id=file_id
        self.file_unique_id = file_unique_id
        self.file_name = file_name
        self.mime_type = mime_type
        self.file_size = file_size

class OpcionDeSalida:
    def __init__(self,iaccionComando:IAccionComando=None):
        self.iaccionComando:IAccionComando=iaccionComando




import os
from telegram import Bot, ParseMode
from telegram import ReplyKeyboardMarkup, ReplyKeyboardRemove
from telegram import InlineKeyboardButton, InlineKeyboardMarkup
from telegram.ext import Updater,Dispatcher
from telegram.ext import CommandHandler, ConversationHandler
from telegram.ext import CallbackQueryHandler, MessageHandler, Filters
from telegram import Update
from telegram.ext import (
    BasePersistence,
    CallbackContext,

    ChosenInlineResultHandler,
    DispatcherHandlerStop,
    Handler,
    InlineQueryHandler,
)
from telegram.ext.utils.types import CCT

from telegram import InlineQueryResultArticle, InputTextMessageContent
#from telegram.ext import InlineQueryHandler

from typing import TYPE_CHECKING, Dict, List, NoReturn, Optional, Union, Tuple, cast, ClassVar
from RenePy.Utiles import *

from ReneTelegramBot.Utiles.TBUtiles import *
from ReneTelegramBot.Utiles.TBUtiles import _enviarTecladoMenu,_banearUsuarioEsteUsuario,_enviarMensaje,_getArgsDeComando,_getUsuarioDeTelegramFrom,_getStrMark,_getTextDeMensaje
from ReneTelegramBot.Clases.TBClases import *

from ReneTelegramBot.Clases.TBClases import _BotonMenuInicial
import re
class SoporteDeBot:
    def estaEnMantenimiento(self)->bool:
        return False
    def accionEnMantenimiento(self,u,c)->None:
        pass
    def estaLogeado(self,u,c)->bool:
        return True
    def loguear(self,u,c):
        pass

class BotConversation:
    def __init__(self, token:str):
        self.updater:Updater = Updater(token=token, use_context=True)
        self.dispatcher:Dispatcher = self.updater.dispatcher
        self.__textoEnMenuDefault=None
        self.textoSiNoEstaLogueadoOAunEnMantenimiento=None

    def setTextoEnMenuDefault(self,textoEnMenuDefault):
        self.__textoEnMenuDefault=textoEnMenuDefault
    def __ponerTextoEnMenuDefault(self,teclado:Teclado):
        # print("poner texto ")
        if isinstance(teclado,Teclado):
            # print("default=",self.__textoEnMenuDefault)
            # print("teclado.textoDeMenu=",teclado.textoDeMenu)
            # print("TEXTO_EN_MENU_DEFAULT=",TEXTO_EN_MENU_DEFAULT)
            # print("(self.__textoEnMenuDefault is not None)=",(self.__textoEnMenuDefault is not None))
            # print("(teclado.textoDeMenu==TEXTO_EN_MENU_DEFAULT)=",teclado.textoDeMenu==TEXTO_EN_MENU_DEFAULT)
            if (self.__textoEnMenuDefault is not None) and teclado.textoDeMenu==TEXTO_EN_MENU_DEFAULT:
                # print("lo puso !!!!!!!!!!!!!!!!!!!!!")
                teclado.textoDeMenu=self.__textoEnMenuDefault
    def __enviarTecladoMenu(self,update,ctx,menuDeBotonesC):
        self.__ponerTextoEnMenuDefault(menuDeBotonesC)
        _enviarTecladoMenu(update, ctx, menuDeBotonesC)

    def __setMenuInicial(self,menuDeBotonesC:MenuDeBotonesC,textoDelBoton:str,metodoComando=None):
        OPCION_MENU_SIGUIENTE = getNumeroOpcionActual()

        def metodoMenuInicial( update,ctx):
            self.__enviarTecladoMenu(update,ctx,menuDeBotonesC)
            #_enviarTecladoMenu(update,ctx,menuDeBotonesC)
            if metodoComando!=None:
                metodoComando(update,ctx)
            return(OPCION_MENU_SIGUIENTE)

        botonMenuInicial = BotonDeMenuC(textoDeboton=textoDelBoton, metodoComando=metodoMenuInicial,
                                        opcionComando=OPCION_MENU_SIGUIENTE)
        self.__botonMenuInicial:_BotonMenuInicial=_BotonMenuInicial(botonMenuInicial,menuDeBotonesC)
        self.__states[OPCION_MENU_SIGUIENTE]=self.__crearHandler(self.__botonMenuInicial.menuDeBotonesC,self.__botonMenuInicial)#update,ctx,

    def sendMensaje(self,chat_id,text):
        try:
            self.updater.bot.sendMessage(chat_id,text)
        except Exception as ex:
            try:
                text=_getStrMark(text)
                self.updater.bot.sendMessage(chat_id,text)
            except Exception as ex:
                try:
                    self.updater.bot.sendMessage(chat_id=chat_id,text= text, parse_mode=ParseMode.HTML)
                except Exception as ex:
                    verException()

    def __getSalidaCorrecta(self,update, ctx,metodo):
        if metodo!=None:
            res = metodo(update, ctx)
            if isinstance(res, OpcionDeSalida):
                iaccion=res.iaccionComando
                if iaccion!=None:
                    return iaccion.metodoComando(update, ctx)
                #_enviarTecladoMenu(update, ctx, self.__botonMenuInicial.menuDeBotonesC)
                self.__enviarTecladoMenu(update, ctx, self.__botonMenuInicial.menuDeBotonesC)
                return self.__botonMenuInicial.opcionComando

            if isinstance(res, IAccionComando):
                if res.metodoComando != None:
                    return res.metodoComando(update, ctx)
            return res
        return None
    def __procesarInputt(self,iParaInputt:IParaInputt,parent:IAccionComando):#,parent#,update,ctx
        inputt:Inputt=iParaInputt.inputt

        accionDelinputt0 = inputt.metodoComando

        def accionConRespuestaDelInput(update, ctx):
            return self.__getSalidaCorrecta(update, ctx,accionDelinputt0)


        inputt.metodoComando = accionConRespuestaDelInput

        if inputt.ponerBotonCancelar:
            textoCancelar="🔙 Cancelar"
            inputt.addTextoAlMenuSoloTextoEnBotones(textoCancelar)


            accionDelinputt=inputt.metodoComando

            def accionConCancelar(update, ctx):
                text: str = update.message.text
                if text == textoCancelar:
                    return(parent.metodoComando(update,ctx))
                elif accionDelinputt!=None:
                    valorRetornado=accionDelinputt(update,ctx)
                    if valorRetornado!=None:
                        return(valorRetornado)
                    else:
                        if isinstance(parent,_BotonMenuInicial):
                            #_enviarTecladoMenu(update, ctx, self.__botonMenuInicial.menuDeBotonesC)
                            self.__enviarTecladoMenu(update, ctx, self.__botonMenuInicial.menuDeBotonesC)
                            return self.__botonMenuInicial.opcionComando
                        else:
                            return iParaInputt.opcionComando
            inputt.metodoComando=accionConCancelar

        accionAllamarDeiParaInputt=iParaInputt.metodoComando

        def accionDeliParaInputt(update, ctx):
            #print("Entra en i para input")
            if inputt.tieneBotones():
                #_enviarTecladoMenu(update, ctx,inputt)
                self.__enviarTecladoMenu(update, ctx,inputt)
                #print("enviar teclado i para input")
            if accionAllamarDeiParaInputt!=None:
                accionAllamarDeiParaInputt(update,ctx)
            return(iParaInputt.opcionComando)

        iParaInputt.metodoComando=accionDeliParaInputt
        filtros=Filters.regex("(.+)")
        if inputt.tipo!=Filters.text:
            filtros=filtros|inputt.tipo
        self.__states[iParaInputt.opcionComando]=[MessageHandler(filtros, inputt.metodoComando)]

        if inputt.hayEncadenados():
            for i in range(len(inputt.encadenados)):
                disparador=inputt.encadenados[i]
                if isinstance(disparador,ITieneMenu) and isinstance(disparador,IAccionComando):
                    self.__ponerIAccion(disparador, iParaInputt)#update,ctx
                    self.__states[disparador.opcionComando] =self.__crearHandler(disparador.menuDeBotonesC,iParaInputt)#update,ctx,
                else:
                    self.__procesarInputt(disparador,iParaInputt)#update,ctx,

    def __ponerIAccion(self,iaccion:IAccionComando,parent):#update,ctx,
        #def crearAccion(iaccion:IAccionComando):
        esITieneMenu: bool = isinstance(iaccion, ITieneMenu)
        if esITieneMenu:
            nuevoBotonAtras = BotonRetroceder(textoDeboton="🔙 Atrás",parent=parent)
            iaccion.menuDeBotonesC.addBoton(nuevoBotonAtras)


            self.__states[iaccion.opcionComando] = self.__crearHandler(iaccion.menuDeBotonesC,iaccion)#update,ctx,

        accionDeBoton=iaccion.metodoComando

        esBotonRetroceder = isinstance(iaccion, BotonRetroceder)
        def accion( update,ctx):
            if esITieneMenu or esBotonRetroceder:
                if iaccion.menuDeBotonesC!=None:
                    #_enviarTecladoMenu( update,ctx, iaccion.menuDeBotonesC)
                    self.__enviarTecladoMenu(update,ctx, iaccion.menuDeBotonesC)
                else:
                    print("atras correspondiente")
                    if esBotonRetroceder and isinstance(iaccion.parent,IParaInputt):
                        iaccion.parent.metodoComando(update,ctx)

            if   (not esBotonRetroceder):#accionDeBoton!=None and
                salida=self.__getSalidaCorrecta(update, ctx,accionDeBoton)
                if salida!=None:
                    return salida

            if esITieneMenu or esBotonRetroceder:
                #return(opcionMenuCorrespondiente)
                return iaccion.opcionComando




            #return(accion)

        #iaccion.metodoComando=crearAccion(iaccion)
        iaccion.metodoComando = accion


    def __crearHandler(self,menuDeBotonesC:MenuDeBotonesC,parent):#update,ctx,
        listaRegexHandler:List[Handler[Update, CCT]] = []

        listaID_Procesados=[]
        def procesar(elemento:ITextAccionComando):
            if isinstance(elemento, IParaInputt):
                self.__procesarInputt( elemento, parent)#update, ctx,
            else:
                self.__ponerIAccion( elemento, parent)#update, ctx,
            listaRegexHandler.append(
                MessageHandler(Filters.regex("^" + elemento.texto + "$"), elemento.metodoComando))


        listaDeBotones=menuDeBotonesC.getListaDeBotones()#update,ctx
        leng=len(listaDeBotones)
        for i in range(leng):
            boton = listaDeBotones[i]
            listaID_Procesados.append(boton.opcionComando)
            procesar(boton)

        comandosInternos=menuDeBotonesC.getComandosInternos()#update,ctx
        leng = len(comandosInternos)
        for i in range(leng):
            comando=comandosInternos[i]
            listaID_Procesados.append(comando.opcionComando)
            procesar(comando)


        if menuDeBotonesC.hayEncadenados():
            for i in range(len(menuDeBotonesC.encadenados)):
                encadenado=menuDeBotonesC.encadenados[i]
                if not contiene(listaID_Procesados,encadenado.opcionComando):
                    procesar(encadenado)


        return(listaRegexHandler)



    def comenzar(self,getBotonInicial,soporteDeBot=None):
        if soporteDeBot is None:
            soporteDeBot=SoporteDeBot()

        STR_PATRON_ID_TELEGRAM="([0-9]{10})"
        PATRON_ID_TELEGRAM=re.compile(STR_PATRON_ID_TELEGRAM)

        if soporteDeBot.estaEnMantenimiento():
            self.dispatcher.add_handler(CommandHandler("start",soporteDeBot.accionEnMantenimiento ))
        else:

            b: BotonParaMenu = getBotonInicial()
            def accionStart(u,c):
                if soporteDeBot.estaEnMantenimiento():
                    return soporteDeBot.accionEnMantenimiento(u,c)
                referencia=None
                a = _getArgsDeComando(c)
                if a != None and len(a) > 0:
                    a = a[0]
                    find = re.findall(PATRON_ID_TELEGRAM, str(a))
                    if find != None and len(find) > 0:
                        #_enviarMensaje(u, c, "referencia es " + str(find[0]))
                        referencia=str(find[0])
                return b.metodoComando(u, c, referencia)

            self.__states: Dict[object, List[Handler[Update, CCT]]] = {}
            self.__setMenuInicial(b.menuDeBotonesC, b.textoDeboton, accionStart)

            handlerMenuInicial=CommandHandler("start", self.__botonMenuInicial.metodoComando)
            conv_handler = ConversationHandler(
                entry_points=[CommandHandler("start", self.__botonMenuInicial.metodoComando, Filters.regex(STR_PATRON_ID_TELEGRAM))
                              ,handlerMenuInicial
                              ],
                states=self.__states,
                fallbacks=[handlerMenuInicial]
            )
            self.dispatcher.add_handler(conv_handler)


            #self.__crearMenu( b.menuDeBotonesC, b.textoDeboton,metodoConComprobarSiHayReferencia)#u, c,  # lambda p_u,p_c:b.metodoComando(p_u, p_c, referencia)

            # self.dispatcher.add_handler(CommandHandler("start", llamarACrearMenu, Filters.regex(STR_PATRON_ID_TELEGRAM)))
            # self.dispatcher.add_handler(CommandHandler("start", llamarACrearMenu))


        def rastreoEnTodosLosMensajes(update, ctx):
            try:
                # print("rastrea este mensaje 1")


                if soporteDeBot.estaEnMantenimiento or not soporteDeBot.estaLogeado(update, ctx):
                    # print("rastrea este mensaje 3_1")
                    text = _getTextDeMensaje(update)
                    if text == r"/start":
                        # print("rastrea este mensaje 4_1")
                        if (not soporteDeBot.estaEnMantenimiento) and (
                                not soporteDeBot.estaLogeado(update, ctx)):
                            # print("rastrea este mensaje 5_1")
                            soporteDeBot.loguear(update, ctx)
                            return None
                    else:
                        # print("rastrea este mensaje 6_1")
                        # _enviarMensaje(update, ctx, "llame al comando "+r"/start")
                        _enviarMensaje(update, ctx, self.textoSiNoEstaLogueadoOAunEnMantenimiento)
                    return ConversationHandler.END
                else:
                    # print("rastrea este mensaje 2_1")
                    pass


                msg = update.message
                if msg.new_chat_members != None:
                    for user in msg.new_chat_members:
                        if user.is_bot:
                            _enviarMensaje(update, ctx, "Va a ser baneado por ser bot")
                            _banearUsuarioEsteUsuario(update)


                # else:
                #     print("rastrea este mensaje 2")
                #     if soporteDeBot.estaEnMantenimiento or not soporteDeBot.estaLogeado(update, ctx):
                #         print("rastrea este mensaje 3")
                #         text=_getTextDeMensaje(update)
                #         if text==r"/start":
                #             print("rastrea este mensaje 4")
                #             if (not soporteDeBot.estaEnMantenimiento) and (not soporteDeBot.estaLogeado(update, ctx)):
                #                 print("rastrea este mensaje 5")
                #                 soporteDeBot.loguear(update, ctx)
                #                 return None
                #         else:
                #             print("rastrea este mensaje 6")
                #             #_enviarMensaje(update, ctx, "llame al comando "+r"/start")
                #             _enviarMensaje(update, ctx, self.textoSiNoEstaLogueadoOAunEnMantenimiento)
                #         return ConversationHandler.END

            except:
                print("rastrea este mensaje error")
                #pass

        self.dispatcher.add_handler(MessageHandler(Filters.regex("(.*)"), rastreoEnTodosLosMensajes))



        self.updater.start_polling(allowed_updates=[])
        self.updater.idle()

        # menuDeBotonesC:MenuDeBotonesC,textoDelBoton:str,metodoComando=None
        # class AlmacenReferencia:
        #     def __init__(self,referencia,idt):
        #         self.referencia=referencia
        #         self.idt=idt
        # def llamarACrearMenu(u,c):#,referencia=None
        #     b:BotonParaMenu=getBotonInicial(u,c)
        #     referencia = AlmacenReferencia(None,str(_getUsuarioDeTelegramFrom(u).id))
        #     a = _getArgsDeComando(c)
        #     if a != None and len(a) > 0:
        #         a = a[0]
        #         find = re.findall(PATRON_ID_TELEGRAM, str(a))
        #         if find != None and len(find) > 0:
        #             _enviarMensaje(u, c, "referencia es " + str(find[0]))
        #             referencia.referencia=str(find[0])
        #             #referencia = AlmacenReferencia(str(find[0]),str(_getUsuarioDeTelegramFrom(u).id))
        #
        #     def metodoConComprobarSiHayReferencia(u,c):
        #
        #         # print("referencia=",referencia.referencia," ut=",referencia.idt)
        #         # _enviarMensaje(u, c, strg("2referencia=",referencia.referencia," ut=",referencia.idt))
        #
        #         return b.metodoComando(u, c, referencia.referencia)
        #
        #
        #     self.__crearMenu(u,c,b.menuDeBotonesC,b.textoDeboton,metodoConComprobarSiHayReferencia)#lambda p_u,p_c:b.metodoComando(p_u, p_c, referencia)


    # def __crearMenu(self,menuDeBotonesC:MenuDeBotonesC,textoDelBoton:str,metodoComando=None):#update,ctx,
    #     self.__states: Dict[object, List[Handler[Update, CCT]]] = {}
    #     self.__setMenuInicial(menuDeBotonesC,textoDelBoton,metodoComando)#update,ctx,
    #     # _enviarTecladoMenu(update,ctx,MenuDeBotonesC(
    #     #     matrizDeBotones=[[BotonDeMenuC(textoDeboton=self.__botonMenuInicial.textoDeboton)]]
    #     #     ,textoDeMenu="Para loquearse escriba *"+self.__botonMenuInicial.textoDeboton+"* o seleccionelo en el menu"
    #     # ))
    #
    #     handlerMenuInicial = MessageHandler((Filters.regex("^" + self.__botonMenuInicial.textoDeboton + "$")),#|Filters.regex("^[/]start$")
    #                                       self.__botonMenuInicial.metodoComando)
    #
    #
    #     conv_handler = ConversationHandler(
    #         entry_points=[handlerMenuInicial],
    #         states=self.__states,
    #         fallbacks=[handlerMenuInicial]
    #     )
    #     self.dispatcher.add_handler(conv_handler)
    #
    #     def rastreoEnTodosLosMensajes(update,ctx):
    #         try:
    #             msg=update.message
    #             if msg.new_chat_members!=None:
    #                 for user in msg.new_chat_members:
    #                     if user.is_bot:
    #                         _enviarMensaje(update,ctx,"Va a ser baneado por ser bot")
    #                         _banearUsuarioEsteUsuario(update)
    #         except:
    #             pass
    #
    #     self.dispatcher.add_handler(MessageHandler(Filters.regex("(.*)"), rastreoEnTodosLosMensajes))
    #



from setuptools import setup

setup(
    name='ReneTelegramBot',
    install_requires=["RenePython==0.9"
                    ,"APScheduler==3.6.3"
                      ,"backports.zoneinfo==0.2.1"
                      ,"cachetools==4.2.2"
                      ,"certifi==2021.10.8"
                      ,"importlib-resources==5.4.0"
                      ,"python-telegram-bot==13.10"
                      ,"pytz==2021.3"
                      ,"pytz-deprecation-shim==0.1.0.post0"
                      ,"six==1.16.0"
                      ,"tornado==6.1"
                      ,"tzdata==2021.5"
                      ,"tzlocal==4.1"
                      ,"zipp==3.6.0"
                      ],
    version='1.1',
    packages=['ReneTelegramBot', 'ReneTelegramBot.Clases', 'ReneTelegramBot.Utiles'],
    description="genial",
    author="René Lázaro Collado Arteaga",
    author_email="renearteaga261998@gmail.com",
    license="GPLv3",
    include_package_data=True,
    classifiers = ["Programming Language :: Python :: 3",
	"License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
	"Development Status :: 4 - Beta", "Intended Audience :: Developers",
	"Operating System :: OS Independent"],
    url="https://github.com/critBus/ReneTelegramBot",
)



from ..Clases.TBClases import *
from telegram.utils import helpers
def _getTextDeMensaje(update):
    return update.message.text
def _getUsuarioDeTelegramFrom(update):
    #user = update.message.from_user
    user = update.effective_message.from_user
    return UsuarioDeTelegram(id=user.id,first_name=user.first_name,last_name=user.last_name
                             ,username=user.username,is_bot=user.is_bot,link=user.link)
def _getChatDeTelgram_DeMensaje(update):
    #chat =update.message.chat
    chat = update.effective_message.chat
    return ChatDeTelegram(id=chat.id,type=chat.type,title=chat.title,first_name=chat.first_name
                          ,last_name=chat.last_name,username=chat.username
                          ,invite_link=chat.invite_link,linked_chat_id=chat.linked_chat_id
                          ,link=chat.link)

def _banearUsuarioEsteUsuario(update):
    user = update.message.from_user
    chat = update.message.chat
    user.decline_join_request(chat.id)
def _deleteMensaje(update):
    update.message.delete()
def _esReplyMensaje(update):
    return update.message.reply_to_message!=None
def _getReplyMensaje(update):
    return update.message.reply_to_message
def _getTextDeReplyMensaje(update):
    return _getReplyMensaje(update).text
def _esDocumento(update):
    return update.message.document!=None
def _descargarDocumento(update,carpeta,nombre):
    url=carpeta+"/"+nombre
    update.message.document.get_file().download(url)
    return url
def _esAudio(update):
    return update.message.audio!=None
def _descargarAudio(update,carpeta,nombre):
    url=carpeta+"/"+nombre
    update.message.audio.get_file().download(url)
    return url
def _getDocumento(update):
    doc=update.message.document
    return DocumentoDeTelgram(file_id=doc.file_id,file_unique_id=doc.file_unique_id,file_name=doc.file_name,mime_type=doc.mime_type,file_size=doc.file_size)

def _tieneFotos(update):
    lf=update.message.photo
    return lf!=None and len(lf)>0
def _descargarFotos(update,carpeta,nombreGenerico):

    lf = update.message.photo
    Lurl=[]
    leng=len(lf)
    for i in range(lf):
        url = carpeta + "/" + nombreGenerico+"%03d"%(i)
        update.message.lf[i].get_file().download(url)
        Lurl.append(url)

    return Lurl

def _esVideo(update):
    return update.message.video!=None
def _descargarVideo(update,carpeta,nombre):
    url=carpeta+"/"+nombre
    update.message.video.get_file().download(url)
    return url

def _esVoz(update):
    return update.message.voice!=None
def _descargarVoz(update,carpeta,nombre):
    url=carpeta+"/"+nombre
    update.message.voice.get_file().download(url)
    return url

def _enviarTecladoMenu( update,ctx,menuDeBotonesC:Teclado):
    reply_keyboard = menuDeBotonesC.getMenuSoloTextoEnBotones(update,ctx)
    # print("se lanza teclado")
    update.message.reply_text(
        menuDeBotonesC.getTextoDeMenu(update,ctx),
        reply_markup=ReplyKeyboardMarkup(reply_keyboard,
                                         one_time_keyboard=False,
                                         resize_keyboard=True))

def _enviarMensaje(update,ctx,mensaje:str,chat_id=None):
    #updater.message.reply_text(mensaje)
    try:
        if chat_id==None:
            try:
                update.message.reply_text(mensaje)
                #update.message.reply_markdown(_getStrMark(mensaje))
            except:
                update.message.reply_text(mensaje)
        else:
            bot=ctx.bot
            try:
                bot.send_message(chat_id,mensaje,parse_mode=ParseMode.MARKDOWN)
            except:
                bot.send_message(chat_id, mensaje)
    except:
        verException()
def _getMessageID(update,ctx):
    return update.message.message_id

def _getArgsDeComando(context):
    return context.args
def _getUrlReferencia(update,context):
    bot = context.bot
    return helpers.create_deep_linked_url(bot.username, str(_getUsuarioDeTelegramFrom(update).id))

def _enviarDoc(update,url,nombre=None):
    arregloBinario=Archivo.readBinaryData(url)
    chat = update.message.chat
    chat.send_document(arregloBinario,nombre)
def _quitarTodosLosComandos(context):
    bot = context.bot
    bot.deleteMyCommands()

def _getStrMark(mensaje):
    return replaceAll(mensaje,"_",r"\_","*",r"\*","[",r"\[","]",r"\]","(",r"\(",")",r"\)","~",r"\~","`",r"\`",">",r"\>","#",r"\#","+",r"\+","-",r"\-","=",r"\=","|",r"\|","{",r"\{","}",r"\}",".",r"\.","!",r"\!")